[RFC PATCH 2/2] backlight: Improve support for dual backlight with primary/secondary linkage

Pengyu Luo mitltlatltl at gmail.com
Sun May 25 10:40:22 UTC 2025


This patch enhances dual-backlight handling by explicitly linking
primary and secondary backlight devices using new fields:

- `is_secondary`: Marks if a device is secondary in a pair
- `secondary`: Points to the associated secondary device (if any)
- `primary`: Points to the primary device (for secondary devices)

It also update `backlight_update_status()` to ensure that both primary
and secondary devices are updated together during brightness changes.
This provides a consistent update mechanism in dual-backlight case.

Suggested-by: Daniel Thompson <danielt at kernel.org>
Signed-off-by: Pengyu Luo <mitltlatltl at gmail.com>
---
 drivers/video/backlight/backlight.c |  9 +++++-
 include/linux/backlight.h           | 50 +++++++++++++++++++++++++++--
 2 files changed, 56 insertions(+), 3 deletions(-)

diff --git a/drivers/video/backlight/backlight.c b/drivers/video/backlight/backlight.c
index 991702f5d..2e7b179bc 100644
--- a/drivers/video/backlight/backlight.c
+++ b/drivers/video/backlight/backlight.c
@@ -388,7 +388,6 @@ struct backlight_device *backlight_device_register(const char *name,
 		new_name = kasprintf(GFP_KERNEL, "%s-secondary", name);
 		if (!new_name)
 			return ERR_PTR(-ENOMEM);
-		put_device(&prev_bd->dev);
 	}
 
 	new_bd->dev.class = &backlight_class;
@@ -428,6 +427,14 @@ struct backlight_device *backlight_device_register(const char *name,
 	list_add(&new_bd->entry, &backlight_dev_list);
 	mutex_unlock(&backlight_dev_list_mutex);
 
+	/* set them until the secondary device is available */
+	if (prev_bd) {
+		prev_bd->secondary = new_bd;
+		new_bd->primary = prev_bd;
+		new_bd->is_secondary = true;
+		put_device(&prev_bd->dev);
+	}
+
 	kfree(new_name);
 
 	return new_bd;
diff --git a/include/linux/backlight.h b/include/linux/backlight.h
index 10e626db7..cde992e10 100644
--- a/include/linux/backlight.h
+++ b/include/linux/backlight.h
@@ -291,13 +291,42 @@ struct backlight_device {
 	 * @use_count: The number of unblanked displays.
 	 */
 	int use_count;
+
+	/**
+	 * @is_secondary: Indicates whether this backlight device is secondary.
+	 */
+	bool is_secondary;
+
+	/**
+	 * @secondary: Pointer to the secondary backlight device.
+	 */
+	struct backlight_device *secondary;
+
+	/**
+	 * @primary: Pointer to the primary backlight device.
+	 *
+	 * Non-NULL only for secondary devices.
+	 */
+	struct backlight_device *primary;
 };
 
+static inline struct backlight_device *
+to_primary_backlight_device(struct backlight_device *bd)
+{
+	return bd->is_secondary ? bd->primary : bd;
+}
+
+static inline struct backlight_device *
+to_secondary_backlight_device(struct backlight_device *bd)
+{
+	return bd->is_secondary ? bd : bd->secondary;
+}
+
 /**
  * backlight_update_status - force an update of the backlight device status
  * @bd: the backlight device
  */
-static inline int backlight_update_status(struct backlight_device *bd)
+static inline int backlight_update_status_single(struct backlight_device *bd)
 {
 	int ret = -ENOENT;
 
@@ -309,6 +338,23 @@ static inline int backlight_update_status(struct backlight_device *bd)
 	return ret;
 }
 
+/**
+ * backlight_update_status - update primary and secondary backlight devices
+ * @bd: the backlight device
+ */
+static inline int backlight_update_status(struct backlight_device *bd)
+{
+	struct backlight_device *primary = to_primary_backlight_device(bd);
+	struct backlight_device *secondary = to_secondary_backlight_device(bd);
+	int ret;
+
+	ret = backlight_update_status_single(primary);
+	if (!secondary || ret)
+		return ret;
+
+	return backlight_update_status_single(secondary);
+}
+
 /**
  * backlight_enable - Enable backlight
  * @bd: the backlight device to enable
@@ -291,13 +291,42 @@ struct backlight_device {
 /**
- * backlight_update_status - force an update of the backlight device status
+ * backlight_update_status_single - force an update of the backlight device status
  * @bd: the backlight device
  */
-- 
2.49.0



More information about the dri-devel mailing list